const cryptojs = require('crypto-js');

const HASH_REGEX = /\b[A-Fa-f0-9]{64}\b/;
const CHUNK_SIZE = 10485760; // 1024 * 1024 * 10
let readingAborted = false;

const sha256 = cryptojs.algo.SHA256.create();
var fileHash = '';

const validateHash = (hash) => {
    return Boolean(hash && hash.match(HASH_REGEX));
}

const arrayBufferToWordArray = (arrayBuffer) => {
    const i8a = new Uint8Array(arrayBuffer);
    const a = [];

    /* tslint:disable:no-magic-numbers */
    for (let i = 0; i < i8a.length; i += 4) {
        a.push((i8a[i] << 24) | (i8a[i + 1] << 16) | (i8a[i + 2] << 8) | i8a[i + 3]);
    }
    /* tslint:enable:no-magic-numbers */

    return cryptojs.lib.WordArray.create(a, i8a.length);
}

const updateHash = (arrayBuffer) => {
    const wordArray = arrayBufferToWordArray(arrayBuffer);
    sha256.update(wordArray);
}

const finalizeHash = () => {
    return sha256.finalize().toString();
}

const initHash = () => {
    return sha256.init();
}

const readChunkedFile = async (
    file,
    processResult,
    uploadProgress,
    readResolve,
    readReject,
) => {
    const chunkNumber = Math.ceil(file.size / CHUNK_SIZE);
    const chunkFactory = (idx) => ({
        offset: CHUNK_SIZE * idx,
        percent: Math.round((idx / chunkNumber) * 100),
    });
    const chunks = _.times(chunkNumber, chunkFactory);

    const processChunk = (chunk) =>
        new Promise(
            (resolve) => {
                const partial = file.slice(chunk.offset, chunk.offset + CHUNK_SIZE);
                const reader = new FileReader();

                reader.onload = () => {
                    processResult(reader.result);
                    resolve();
                };

                reader.onerror = () => {
                    reader.abort();
                    readReject();
                };

                reader.readAsArrayBuffer(partial);
            },
        );

    for (let chunk of chunks) {
        runtime.variable.set(uploadProgress, {
            percents: chunk.percent,
            label: `${chunk.percent}%`,
        });

        await processChunk(chunk);

        if (readingAborted) {
            readingAborted = false;
            readReject();

            return;
        }
    }

    readResolve();
};

const abortCalculateHash = () => {
    readingAborted = true;
};

const processResult = (data) => {
    updateHash(data);
};

const resolve = () => {
    fileHash = finalizeHash().toUpperCase();
//    fileHash = sha256.finalize().toString();
//    sha256.init();
};

const reject = () => {
    fileHash = '';
};

async function CalcSha256(file) {
    initHash();
    await readChunkedFile(file, processResult, 'fileUploadProgress', resolve, reject);
    return fileHash;
}

module.exports = {
    CalcSha256,
    abortCalculateHash,
    finalizeHash
}