const lo = require('lodash');
const hlp = require('../../../helpers');
const hlpPolicy = require('../../../helpers_policy');
const utilsOas = require('../../utils/oas.js');
const hlpSch = require('./TaskScheduleHelper.js');

const defPolicySections = ['OasPolicy','PredefinedTasksPolicy'];
const formCommonValues = ['ScanByAccessType','UseHeuristicAnalyzer','HeuristicAnalyzerLevel','UseTrustedZoneSettings',
    'UseKsn','UseHostBlocker','UseSandbox','UseActiveDisinfection','UseDeepcheckOnProcessStart'];
const formScopeValues = ['ScanScope', 'ExcludedFromScanScope'];

function toView(TASK_ADDITIONAL_PARAMS, activePolicyCache, formType) {
    if(formType === 'Schedule' ) {
        return hlpSch.toView(TASK_ADDITIONAL_PARAMS.Schedule, activePolicyCache.PredefinedTasksPolicy.OasTaskStartOptions);
    }
    if(formType!=='Main')
        return {};

    let data = {
        Form_Protection_RtpCommon: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.OasPolicy, formCommonValues),
        Form_Protection_RtpScope: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.OasPolicy, formScopeValues)
    }
    data.Form_Protection_RtpScope = utilsOas.toViewScanScope(data.Form_Protection_RtpScope);
    data.Form_Protection_RtpCommon.LocalTask = true;
    data.Form_Protection_RtpScope.LocalTask = true;
    return data;
}

function toRaw(TASK_ADDITIONAL_PARAMS, changedData, formType) {
    if(formType === 'Schedule' ) {
        hlpSch.toRaw(changedData, TASK_ADDITIONAL_PARAMS.Schedule);
        return;
    }
    else if(formType==='Main') {
        // common
        const Common = hlp.ItemToLocalSettings(changedData.Form_Protection_RtpCommon);
        Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(Common, formCommonValues));
        // scope
        const ScanScope = utilsOas.toRowSettingsScanScope(changedData.Form_Protection_RtpScope);
        Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(ScanScope, formScopeValues));
    }
}

function RbacToPolicyCache(activePolicyCache){
    const allFields = formCommonValues.concat(formScopeValues);
    activePolicyCache.OasPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.OasPolicy, allFields);
    activePolicyCache.PredefinedTasksPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.PredefinedTasksPolicy, ['OasTaskStartOptions']);
    return activePolicyCache;
}

module.exports = {
    defPolicySections,
    toView,
    toRaw,
    RbacToPolicyCache
};
