const fs = require('fs');
const path = require('path');
const hlpDef = require('../../default/policy.js');
const utils = require('../../utils.js');
const utilsCmn = require('./common.js');

let _locEvents = {};

function readAlertsLocalization(loc) {
    try {
        const filePath = path.resolve(__dirname, `../../localization/alerts/${loc}.json`);
        const out = fs.readFileSync(filePath, 'utf8');
        return out!==undefined ? JSON.parse(out) : {};
    } catch (error) {
        console.log(error);
    }
    return {};
}

function GetAlert(eventId, severity, locEvents) {
    let alert = hlpDef.defaultEventsAlertOption();
    alert.EventType.KLPRSS_Val = eventId;
    alert.EventSeverity.KLPRSS_Val = severity;

    if(_locEvents!==undefined ){
        const ev_data =  _locEvents[eventId]!==undefined ? _locEvents[eventId] : _locEvents.def;
        const pref =  _locEvents.pref!==undefined ? _locEvents.pref : '';
        if( ev_data.AdminMessage!==undefined )
            alert.AdminMessage.KLPRSS_Val = pref + ' ' + ev_data.AdminMessage;
        if( ev_data.UserMessage!==undefined )
            alert.UserMessage.KLPRSS_Val = pref + ' ' + ev_data.UserMessage;
    }
    return alert;
}

function CreateDefaultEvents(loc) {
    _locEvents =  readAlertsLocalization(loc!==undefined ? loc : 'en-us');

    let alerts = [];
    alerts.push(GetAlert(4111,2));
    alerts.push(GetAlert(8194,1));
    alerts.push(GetAlert(4196,1));
    alerts.push(GetAlert(4101,3));
    alerts.push(GetAlert(4102,3));
    alerts.push(GetAlert(8227,2));
    alerts.push(GetAlert(8202,1));
    alerts.push(GetAlert(8204,2));
    alerts.push(GetAlert(8206,2));
    alerts.push(GetAlert(8200,1));
    alerts.push(GetAlert(4136,2));
    alerts.push(GetAlert(4144,2));
    alerts.push(GetAlert(4135,1));
    alerts.push(GetAlert(4143,1));
    alerts.push(GetAlert(4197,2));
    alerts.push(GetAlert(4866,2));
    alerts.push(GetAlert(8210,1));
    alerts.push(GetAlert(4110,1));
    alerts.push(GetAlert(8240,2));
    alerts.push(GetAlert(8241,2));
    alerts.push(GetAlert(4115,1));
    alerts.push(GetAlert(8289,1));
    alerts.push(GetAlert(8295,1));
    alerts.push(GetAlert(8291,2));
    alerts.push(GetAlert(8264,1));
    alerts.push(GetAlert(8269,1));
    alerts.push(GetAlert(8267,2));
    alerts.push(GetAlert(4160,2));
    alerts.push(GetAlert(8277,2));
    alerts.push(GetAlert(8512,1));
    alerts.push(GetAlert(8513,1));
    alerts.push(GetAlert(8514,1));
    alerts.push(GetAlert(8515,1));
    alerts.push(GetAlert(8516,1));
    alerts.push(GetAlert(8517,1));
    alerts.push(GetAlert(8519,1));
    alerts.push(GetAlert(8520,1));

    return alerts;
}

function AlertToView(evt) {
    evt.uiFields = {
        AdminNetsendRecipients: evt.AdminNetsendSettings.KLPRSS_Val.Recipients.KLPRSS_Val.join(),
        AdminEmailRecipients: evt.EmailSettings.KLPRSS_Val.Recipients.KLPRSS_Val.join()
    };
    // cmd exec password
    const curCmdExecPswrd = evt.CmdExecSettings.KLPRSS_Val.ImpersonationPassword.KLPRSS_Val.value;
    evt.NewCmdExecPassword = '';
    evt.UseEmptyCmdExecPassword = (curCmdExecPswrd==='') ? true : false;
    evt.CmdExecPswrdWasSet = (curCmdExecPswrd==='') ? false : true;
    // email password
    const curSmtpPswrd = evt.EmailSettings.KLPRSS_Val.SmtpServerAuthPassword.KLPRSS_Val.value;
    evt.NewSmtpPassword = '';
    evt.UseEmptySmtpPassword = (curSmtpPswrd==='') ? true : false;
    evt.SmtpPswrdWasSet = (curSmtpPswrd==='') ? false : true;
    return evt;
}

async function AlertToRow(kscApi, evt) {
    evt.AdminNetsendSettings.KLPRSS_Val.Recipients.KLPRSS_Val = utils.normalizedArrayMasks(evt.uiFields.AdminNetsendRecipients);
    evt.EmailSettings.KLPRSS_Val.Recipients.KLPRSS_Val = utils.normalizedArrayMasks(evt.uiFields.AdminEmailRecipients);
    // cmd exec password
    if(evt.UseEmptyCmdExecPassword){
        evt.CmdExecSettings.KLPRSS_Val.ImpersonationPassword.KLPRSS_Val.value = '';
    }
    else if(evt.NewCmdExecPassword!=='') {
        const pswrdCmdExec = utilsCmn.getPswdrData(evt.NewCmdExecPassword);
        const response = await kscApi.DataProtectionApi.ProtectDataGlobally({ pData:pswrdCmdExec })
        if(response.result!=='')
            evt.CmdExecSettings.KLPRSS_Val.ImpersonationPassword.KLPRSS_Val.value = response.result;
    }
    delete evt.UseEmptyCmdExecPassword;
    delete evt.NewCmdExecPassword;
    delete evt.CmdExecPswrdWasSet;

    // smtp
    if(evt.UseEmptySmtpPassword){
        evt.EmailSettings.KLPRSS_Val.SmtpServerAuthPassword.KLPRSS_Val.value = '';
    }
    else if(evt.NewSmtpPassword!=='') {
        const pswrdSmtp = utilsCmn.getPswdrData(evt.NewSmtpPassword);
        const response = await kscApi.DataProtectionApi.ProtectDataGlobally({ pData:pswrdSmtp })
        if(response.result!=='')
            evt.EmailSettings.KLPRSS_Val.SmtpServerAuthPassword.KLPRSS_Val.value = response.result;
    }
    delete evt.UseEmptySmtpPassword;
    delete evt.NewSmtpPassword;
    delete evt.SmtpPswrdWasSet;

    return evt;
}

module.exports = {
    CreateDefaultEvents,
    AlertToView,
    AlertToRow
}